// ArduinoJson - Version: 6.16.0
#include <ArduinoJson.h>
#include <ArduinoJson.hpp>

// Set configuration options for pins, WiFi, atd. in the following file:
#include "config.h"

// https://github.com/bblanchon/ArduinoJson
#include <ArduinoJson.h>

// Library to work with Wifi
#include <ESP8266WiFi.h>
#include <ESP8266HTTPClient.h>
#include <WiFiClient.h>

void setup(){
  // Open serial communications and wait for port to open (Debug):
  if (CONFIG_DEBUG){
    Serial.begin(9600);
  }
 
  // Set Pin like a input
  pinMode(CONFIG_IN_GATE_PIN, INPUT_PULLUP);
  pinMode(CONFIG_OUT_GATE_PIN, INPUT_PULLUP);

  // Start Wifi
  setup_wifi();
}

void setup_wifi(){
  delay(10);
  
  // We start by connecting to a WiFi network
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(CONFIG_WIFI_SSID);

  // Disable the built-in WiFi access point.
  WiFi.mode(WIFI_STA);
  
  // Try connect to WiFi
  WiFi.begin(CONFIG_WIFI_SSID, CONFIG_WIFI_PASS);

  // Try conecting to WiFi until sucess
  while (WiFi.status() != WL_CONNECTED){
    delay(500);
    Serial.print(".");
  }

  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
}

void loop(){
  // Incaming counter, if first gate is reach
  if (digitalRead(CONFIG_IN_GATE_PIN)){
      Serial.println("Income 1/2");
      
    // Variable to count leinght of walkthru measurements 
    int time = 1;
    
    // Try to read data from secon gate for 'CONFIG_TIME_DELAY'
    while (time < CONFIG_TIME_DELAY){
      delay(1);
      // and if gate is reach       
      if (digitalRead(CONFIG_OUT_GATE_PIN)){
        Serial.println("Income 2/2");
        // send data to server
        sendData(1, in);
        loop();
      }
      
      time = time + 1;
    }
  }

  // Outcaming counter, if first gate is reach
  if (digitalRead(CONFIG_OUT_GATE_PIN)){
    Serial.println("Outcome 1/2");
    
    // variable to count leinght of walkthru measurements 
    int time = 1;
    
    while (time < CONFIG_TIME_DELAY){
      delay(1);
      
      // and if gate is reach 
      if (digitalRead(CONFIG_IN_GATE_PIN)){
        Serial.println("Outcome 2/2");
        // send data to server
        sendData(-1, out);
        loop();
      }
      
      time = time + 1;
    }
  }

  delay(1000);
}

// Function to upload data to server
// Passage (coming) = type of passage (in/out)
void sendData(int count, coming passagee){
  // Check WiFi connection status
  if(WiFi.status()== WL_CONNECTED){
    WiFiClient client;
    HTTPClient http;
    
    // Your Domain name with URL path or IP address with path
    http.begin(client, CONFIG_BASE_URL);
    http.addHeader("Content-Type", "application/json");
    http.addHeader("TOKEN", CONFIG_TOKEN);

    StaticJsonDocument<200> doc;

    // Add values in the document
    doc["type"] = cominStr[passagee];
    doc["count"] = count;
    doc["room"] = CONFIG_ROOM;

    // Generate the prettified JSON and send it to the Serial port.
    char json[100];

    serializeJsonPretty(doc, json);

    // Send HTTP POST request
    int httpResponseCode = http.POST(json);
    
    // Validate error callback
    if (httpResponseCode>0){
      Serial.print("HTTP Response code: ");
      Serial.println(httpResponseCode);
      String payload = http.getString();
      Serial.println(payload);
    }else{
      Serial.print("Error code: ");
      Serial.println(httpResponseCode);
    }
    
    // Free resources
    http.end();
    
    delay(1000);
  }else{
    Serial.println("WiFi Disconnected");
  }
}