// ArduinoJson - Version: 6.16.0
#include <ArduinoJson.h>
#include <ArduinoJson.hpp>

// https://github.com/bblanchon/ArduinoJson
#include <ArduinoJson.h>
#include "functions.h"

#define disable 0
#define enable 1

unsigned int channel = 1;

void setup(){
  // Open serial communications and wait for port to open (Debug):
  if (CONFIG_DEBUG){
    Serial.begin(9600);
  }
  
  delay(500);
  Serial.println(F("Wifi Sniffing"));
}

void loop(){
  // Setup device to search for mac adress
  setup_sniffer();

  // Move thrue 15 channels
  channel = 1;
  wifi_set_channel(channel);
  
  while (true){
    nothing_new++;
    
    if (nothing_new > 200){
      nothing_new = 0;
      channel++;
      if (channel == 15) break;
      wifi_set_channel(channel);
    }
    
    delay(1);
  }

  // Validate if object is not empty
  bool skip = true;

  // Create JSON documnt
  StaticJsonDocument<3000> doc;
  JsonArray array = doc.to<JsonArray>();

  // For loop all find clients
  for (int f = 0; f < clients_known_count; f++){
    clientinfo ci = clients_known[f];

    // If client not empty or corupt
    if (ci.err == 0 || ci.rssi != 0){
      skip = false;

      // Create client object and fill data
      JsonObject object = array.createNestedObject();
      object["room"] = CONFIG_ROOM;

      // Mac adress is saved like array of byts 
      String mac = "";
      for (int i = 0; i < 6; i++){
        mac = mac + ci.station[i];
      }
      
      object["mac"] = mac;

      // Mac adress is saved like array of byts 
      String ap_mac = "";
      for (int i = 0; i < 6; i++){
        ap_mac = ap_mac + ci.ap[i];
      }
      
      object["ap_mac"] = ap_mac;
      object["rssi"] = ci.rssi;

      // Add info about AP saved in separed variable on device
      for (int u = 0; u < aps_known_count; u++){
        if (!memcmp(aps_known[u].bssid, ci.bssid, ETH_MAC_LEN)){
          object["ap_ssid"] = aps_known[u].ssid;
          object["chanel"] = aps_known[u].channel;
          break;
        }
      }
    }
  }

  // If "skip" var used to procet send empty request
  if (skip){
    Serial.println("Empty JSON");
  }else{
    // Generate the prettified JSON and send it to the Serial port.
    char json[3000];
    serializeJsonPretty(doc, json);

    Serial.println(json);
    delay(1000);
    
    // Setup wifi
    setup_wifi_conection();
    delay(1000);
    
    // Send data serialized in json var
    send_data(json);
    
    // Wait for 6 min
    delay(60000);
  }
  
  // clear saved clients and data on device
  clear_data();
}

void setup_sniffer(){
  wifi_set_opmode(STATION_MODE);
  wifi_set_channel(channel);
  wifi_promiscuous_enable(disable);
  
  //this line enable sniffing onli thing you neen to do is change chanel
  wifi_set_promiscuous_rx_cb(promisc_cb);
  wifi_promiscuous_enable(enable);
}

void setup_wifi_conection(){
  delay(10);
  
  // We start by connecting to a WiFi network
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(CONFIG_WIFI_SSID);

  wifi_promiscuous_enable(disable);
  
  // Try connect to WiFi
  WiFi.begin(CONFIG_WIFI_SSID, CONFIG_WIFI_PASS);

  // Try conecting to WiFi until sucess
  while (WiFi.status() != WL_CONNECTED){
    delay(500);
    Serial.print(".");
  }

  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
}